import java.util.Set;
import java.util.Map;
import java.util.TreeMap;
import java.util.List;
import java.util.LinkedList;
import java.util.Arrays;
import com.google.common.graph.Network;
import com.google.common.graph.NetworkBuilder;
import com.google.common.graph.ImmutableNetwork;
import com.google.common.graph.MutableNetwork;
import com.google.common.graph.EndpointPair;
import java.lang.IllegalArgumentException;

// See https://google.github.io/styleguide/javaguide.html

/** This class solves the traveling salesperson problem on a graph. */
public class TSP {

	
	public static void main(String[] args) throws IllegalArgumentException { 
        System.out.println("------------------- TSP Version 1: First Pass Algorithm ----------------------");
		int N;
		if(args.length==1) {
			N = Integer.decode(args[0]);
		} else {
			N = 5;
		}
		double conn = 1.00;
		TSP t = new TSP(N,conn);

        long start = System.nanoTime();
		t.solve();
        long end = System.nanoTime();
        long duration = end - start;
        System.out.printf("Found solution...?\n");
        System.out.printf("Elapsed time %03f s\n ", (duration/1E9) );
	}


	/**
	 * This class solves the traveling salesperson problem on a graph.
	 * It makes use of Google's Guava library.
	 * This implements a recursive backtracking solution
	 * to search for the shortest path.
	 * 
	 * As is, this class hard-codes an adjacency matrix 
	 * for a network of 5 cities.
	 */

	// The actual graph of cities
	ImmutableNetwork<Node,Edge> graph;
	int graphSize;

	// Storage variables used when searching for a solution 
	int[] route;			// store the route
	int this_distance;	// store the total distance
	int min_distance;	// store the shortest path found so far

	/** Default constructor generates the graph and initializes storage variables */
	public TSP(int N, double connectivity) throws IllegalArgumentException {

        // In case we do something stupid
        if(connectivity>1.0 || connectivity<0.0) { 
            throw new IllegalArgumentException("Connectivity value must be between 0 and 1.");
        }

		// Build the graph
		this.graph = RandomGraph.buildGraph(N, connectivity);
		RandomGraph.toDot(this.graph);
		this.graphSize = N;
		
		// Initialize route variable, shared across recursive method instances
		this.route = new int[this.graphSize];
		for(int j=0; j<this.graphSize; j++) { 
			// -1 means no node selected
			this.route[j] = -1;
		}
		
		// Initialize distance variable, shared across recursive method instances
		this.this_distance = 0;
		this.min_distance = -1; // negative min means uninitialized
	}

	
	/** Public solve method will call the recursive backtracking method to search for solutions on the graph */
	public void solve() {
		/** To solve the traveling salesperson problem:
		 * Set up the graph, choose a starting node, then call the recursive backtracking method and pass it the starting node.
		 */

		// We need to pass a starting node to recursive backtracking method
		Node startNode = null;
		
		// Grab a node, any node...
		for( Node n : graph.nodes() ) {
			startNode = n;
			break;
		}
		
		// Visit the first node
		startNode.visit();
		
		// Add first node to the route
		this.route[0] = startNode.id;
		
		// Pass the number of choices made
		int nchoices = 1;
		
		// Recursive backtracking
		explore(startNode, nchoices);
	}
	
	/** Recursive backtracking method: explore possible solutions starting at this node, having made nchoices */
	public void explore(Node node, int nchoices) {
		/**
		 * Solution strategy: recursive backtracking.
		 *
		 * Base case:
		 *		- We've visited as many cities as are on the graph.
		 *		- Check if this is a new solution (distance less than the current minimum).
		 *
		 *	Recursive case:
		 *		- Make a choice (mark node as visited, add city to route).
		 *		- Explore the consequences (recursive call).
		 *		- Unmake the choice (mark node as unvisited, remove city from route).
		 *		- Move on to next choice.
		 */

		if(nchoices == graphSize) {
			// 
			// BASE CASE
			//
			if(this.this_distance < this.min_distance || this.min_distance < 0) {
				// Solution base case:
				// if this_distance < min_distance, this is our new minimum distance
				// if min_distance < 0, this is our first minimium distance
				this.min_distance = this.this_distance;
				printSolution();
			} else {
				printFailure();
			}
			
		} else {
			//
			// RECURSIVE CASE
			// 	
			Set<Node> neighbors = graph.adjacentNodes(node);
			for(Node neighbor : neighbors) {
				if(neighbor.visited == false) {
					
					int distance_btwn = -10000;
					
					// One edge
					for( Edge edge : graph.edgesConnecting(node, neighbor) ) {
						distance_btwn = edge.value;
					}

					// Make a choice
					this.route[nchoices] = neighbor.id;
					neighbor.visit();
					this.this_distance += distance_btwn;
					
					// Explore the consequences
					explore(neighbor,nchoices+1);
					
					// Unmake the choice
					this.route[nchoices] = -1;
					neighbor.unvisit();
					this.this_distance -= distance_btwn;
				}
				// Move on to the next choice (continue loop)
			}				
		} // End base/recursive case
	}

	/** Print out solution */
	public void printSolution() {
		//System.out.print("!!!!!YAY!!!!!!\tNEW SOLUTION\t");
		//System.out.println("Route: "+Arrays.toString(this.route)
		//				  +"\tDistance: "+this.min_distance);
		System.out.print(".");
	}
	/** Print out failed path */
	public void printFailure() { }
}

/** Node class to represent cities */
class Node {
	public int id;
	public boolean visited; // Helps us to keep track of where we've been on the graph
	public Node(int id){
		this.id = id;
		this.visited = false;
	}
	public void visit(){
		this.visited = true;
	}
	public void unvisit() {
		this.visited = false;
	}
}

/** Edge class to represent roads connecting cities */
class Edge {
	public int value;
	public Edge(int value) {
		this.value = value;
	}
}
