import java.util.Random;
import java.util.Set;
import java.util.Map;
import java.util.TreeMap;
import java.util.List;
import java.util.LinkedList;
import java.util.Arrays;
import java.util.Scanner;
import java.io.File;
import java.io.PrintStream;
import com.google.common.graph.Network;
import com.google.common.graph.NetworkBuilder;
import com.google.common.graph.ImmutableNetwork;
import com.google.common.graph.MutableNetwork;
import com.google.common.graph.EndpointPair;
import java.lang.IllegalArgumentException;

/** Static class to generate random graphs with values at the nodes.
 */
public class RandomNodeGraph { 
	private RandomNodeGraph(){
	};

	private static int getNextCost(int T) {
		Random r = new Random();
		return 1+r.nextInt(T);
	}

	private static int getNextPay(int T) {
		Random r = new Random();
		return (int)(0.5*T + 0.5*(r.nextInt(T)+1));
	}

	private static String getWeird(int id) {
		try {
			Scanner s = new Scanner(new File("silly-names.dat"));
			for(int i = 0; i < id; i++) { 
				if(s.hasNext()) {
					s.next();
				}
			}
			String result = "";
			if(s.hasNext()) {
				result = s.next();
			}
			return result;
		} catch (Exception e) {
			return "";
		}
	}

	
	public static ImmutableNetwork<Node,Edge> buildGraph(int nodes, int tank) throws IllegalArgumentException { 
		// If connectivity not specified, set it to 1.
		return buildGraph(nodes,tank,(nodes*(nodes-1))/2);
	}

	public static ImmutableNetwork<Node,Edge> buildGraph(int nodes, int tank, double connectivity) throws IllegalArgumentException { 
		// If connectivity not specified, set it to 1.
		return buildGraph(nodes,tank,(int)connectivity*(nodes*(nodes-1))/2);
	}
		
	public static ImmutableNetwork<Node,Edge> buildGraph(int N, int T, int E ) throws IllegalArgumentException {

		// This is multiplied by the gas tank's capacity
		// to set the maximum distance an edge can have.
		double sweat_factor = 1.0;

		// This is multiplied by the gas tank's capacity
		// to set the maximum pay a node can have.
		double pay_factor = 1.1;

		// Make a NetworkBuilder object for an undirected network and call build().
        //
        // https://github.com/google/guava/wiki/GraphsExplained#building-graph-instances
		//
		// This will load an adjacency matrix from a file.
		//
		// nodes is the number of nodes on the graph.
		// connectivity is a number between 0 (totally disconnected) and 1 (fully connected).

		// MutableNetwork is an interface requiring a type for nodes and a type for edges
		MutableNetwork<Node,Edge> roads = NetworkBuilder.undirected().build();

		// Add nodes to graph
		List<Node> all_nodes = new LinkedList<Node>();
		for(int n=0; n<N; n++) {
			int pay = getNextPay(T);
			Node thenode = new Node(n,pay);
			all_nodes.add(thenode);
			roads.addNode(thenode);
		}

		// Adding random edges to graph
		// between n_i and n_j
		//
		// If i != j, and edges are single and symmetric, 
		// N nodes have N! possible connections.
		//
		// start with a sequence enumerating all possible edges of the graph:
		// j = 1 ... N!
		// Now we can convert each enumeration to its corresponding pair 
		//
		//
		class Pair {
			int left;
			int right;
			public Pair(int left, int right) { 
				this.left = left;
				this.right = right;
			}
		}
		// Make 2 arrays: 
		// one integers ix = 1 .. E
		// one pairs P(i,j), i != j
		// we will make all possible pairs,
		// then shuffle indexes to pick pairs at random.
		Pair[] all_edges = new Pair[(N*(N-1))/2];
		int[] index = new int[(N*(N-1))/2];
		int c = 0;
		for(int i=0;i<N;i++){ 
			for(int j=i+1;j<N;j++){
				// make the pair and increment the index
				all_edges[c] = new Pair(i,j);
				index[c] = c;
				c++;
			}
		}
		// mix up the pair indexes
		RandomArray.shuffle(index);
		for(int e : index) { 
			Pair p = all_edges[e];

			// now we have our random pair.
			// create a random distance and add to the graph.
			int cost = getNextCost(T);
			Edge edg = new Edge(cost);
			roads.addEdge(all_nodes.get(p.left),all_nodes.get(p.right),edg);
		}

		// freeze the network
		ImmutableNetwork<Node,Edge> frozen_roads = ImmutableNetwork.copyOf(roads);
		
		return frozen_roads;
	}

	public static void toDot( ImmutableNetwork<Node,Edge> graph) {
		String dot = "graph G{\n";
		dot += "\tratio = fill; node [shape = circle]; edge [dir=none];\n";
		int i = 0;
		for(Node n : graph.nodes()) {
			dot += "\t" + n.id + " [style=filled fillcolor=gray label = \""+ getWeird(n.id) + " (+" + n.pay + " gal)\"];\n";
		}
		int j = 0;
		for(Edge e : graph.edges()) { 
			EndpointPair<Node> pair = graph.incidentNodes(e);
			Node left = pair.nodeU();
			Node right = pair.nodeV();
			// gas cost:
			dot += "\t" + left.id + " -- " + right.id + " [label = \"" + e.cost + " gal\"];\n";

			//// distance:
			//dot += "\t" + left.id + " -- " + right.id + " [label = \"" + e.distance + "\"];\n";
		}
		dot += "}\n";
		try{ 
			PrintStream p = new PrintStream(new File("graphviz.dot"));
			p.println(dot);
		} catch(Exception e) {
			// meh.
		}
	}
}

/** Static class to shuffle an array.
 * (That's Java for you.)
 */
class RandomArray { 
	public static void shuffle(int[] arr) { 
		Random r = new Random(300);
		for(int i=(arr.length-1); i>0; i--) { 
			int j = r.nextInt(i+1);
			// swap arr[i] and arr[j]
			int temp = arr[i];
			arr[i] = arr[j];
			arr[j] = temp;
		}
	}
}


