import java.io.File;
import java.io.FileNotFoundException;
import java.lang.IllegalArgumentException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeMap;
import com.google.common.graph.EndpointPair;
import com.google.common.graph.ImmutableNetwork;
import com.google.common.graph.MutableNetwork;
import com.google.common.graph.Network;
import com.google.common.graph.NetworkBuilder;

// See https://google.github.io/styleguide/javaguide.html

/** 
 * This class solves the traveling schoolteacher problem on a graph. 
 * @class TSTP 
 * 
 * The traveling schoolteacher problem is a constrained traveling salesperson problem.
 * The constraint is, the salesperson is now a poor schoolteacher with an old car,
 * and a limited gas budget.
 *
 * This class makes use of:
 * - Google Guava for representing the graph using a Network object
 * - Recursive backtracking for finding the shortest path
 * - Randomly generated graph
 */
public class TSTP {

	public static void main(String[] args) throws IllegalArgumentException { 
        System.out.println("------------------- Traveling Schoolteacher Problem ----------------------");
		int N; // number of nodes
		int T; // gas tank capacity
		if(args.length==2) {
			N = Integer.decode(args[0]);
			T = Integer.decode(args[0]);
		} else {
			N = 12;
			T = 20;
		}
		System.out.println("Number of nodes\tN = "+N);
		System.out.println("Gas tank size\tT = "+T);
		double conn = 1.00;
		TSTP t = new TSTP(N,T,conn);

        long start = System.nanoTime();
		t.solve();
        long end = System.nanoTime();
        long duration = end - start;
        System.out.printf("Found solution...?\n");
        System.out.printf("Elapsed time %03f s\n ", (duration/1E9) );
	}

	// The actual graph of cities
	ImmutableNetwork<Node,Edge> graph;
	int graphSize;
	int tankSize;

	// Storage variables used when searching for a solution 
	int[] route;		// store the route
	int this_distance;	// store the total distance
	int min_distance;	// store the shortest path found so far
	int gas_tank;		// store the current gas tank state

	/** Default constructor generates the graph and initializes storage variables */
	public TSTP(int N, int T, double connectivity) throws IllegalArgumentException {

        // In case we do something stupid
        if(connectivity>1.0 || connectivity<0.0) { 
            throw new IllegalArgumentException("Connectivity value must be between 0 and 1.");
        }

		// Build the graph
		this.graph = RandomNodeGraph.buildGraph(N, T, connectivity);
		RandomNodeGraph.toDot(this.graph);
		this.graphSize = N;
		this.tankSize = T;
		
		// Initialize route variable, shared across recursive method instances
		this.route = new int[this.graphSize];
		for(int j=0; j<this.graphSize; j++) { 
			// -1 means no node selected
			this.route[j] = -1;
		}
		
	}

	/** Public solve method will call the recursive backtracking method to search for solutions on the graph */
	public void solve() {
		// To solve the traveling salesperson problem:
		// Set up the graph, choose a starting node, then call the recursive backtracking method and pass it the starting node.

		// We need to pass a starting node to recursive backtracking method
		Node startNode = null;
		
		// Grab a node, any node...
		for( Node n : graph.nodes() ) {
			startNode = n;
			break;
		}
		
		// Visit the first node
		startNode.visit();
		
		// Add first node to the route
		this.route[0] = startNode.id;
		
		// Pass the number of choices made
		int nchoices = 1;
		
		// Initialize other "global" solution variables:

		// Initialize distance variable, shared across recursive method instances
		this.this_distance = 0;
		this.min_distance = -1; // negative min means uninitialized

		// Full tank of gas, 
		// half a pack of cigarettes,
		// it's dark,
		// and we're wearing sunglasses.
		this.gas_tank = this.tankSize;
		// Hit it!

		// Recursive backtracking
		explore(startNode, nchoices);
	}
	
	/** Recursive backtracking method: explore possible solutions starting at this node, having made nchoices */
	public void explore(Node node, int nchoices) {

		 // Solution strategy: recursive backtracking.
		 //
		 // Base case:
		 //		- We've visited as many cities as are on the graph.
		 //		- Check if this is a new solution (distance less than the current minimum).
		 //
		 //	Recursive case:
		 //		- Make a choice (mark node as visited, add city to route).
		 //		- Explore the consequences (recursive call).
		 //		- Unmake the choice (mark node as unvisited, remove city from route).
		 //		- Move on to next choice.

		if(nchoices == graphSize) {
			// 
			// BASE CASE
			//
			if(this.this_distance < this.min_distance || this.min_distance < 0) {
				// Solution base case:
				// if this_distance < min_distance, this is our new minimum distance
				// if min_distance < 0, this is our first minimium distance
				this.min_distance = this.this_distance;
				printSolution();
			} else {
				printFailure();
			}
			
		} else {
			//
			// RECURSIVE CASE
			// 	
			if(this.gas_tank <= 0) {
				// Bummer, man.
				return;
			}
			if(this.min_distance > 0 && this.this_distance > this.min_distance) {
				// Give up, there's no hope.
				return;
			}

			// Now the teacher teaches,
			// Now the teacher gets some gas. 
			// If the tank is full...  bummer, man.
			this.gas_tank += Math.min(this.tankSize, this.gas_tank + node.pay);

			Set<Node> neighbors = graph.adjacentNodes(node);
			for(Node neighbor : neighbors) {
				if(neighbor.visited == false) {
					
					int distance_btwn = -10000;
					
					// One edge
					for( Edge edge : graph.edgesConnecting(node, neighbor) ) {
						distance_btwn = edge.cost;
					}

					// Make a choice
					this.route[nchoices] = neighbor.id;
					neighbor.visit();
					this.this_distance += distance_btwn;
					
					// Explore the consequences
					explore(neighbor,nchoices+1);
					
					// Unmake the choice
					this.route[nchoices] = -1;
					neighbor.unvisit();
					this.this_distance -= distance_btwn;
				}
				// Move on to the next choice (continue loop)
			}				
		} // End base/recursive case
	}

	/** Print out solution */
	public void printSolution() {
		System.out.print("!!!!!YAY!!!!!!\tNEW SOLUTION\t");
		System.out.println("Route: "+Arrays.toString(this.route)
						  +"\tDistance: "+this.min_distance);
	}
	/** Print out failed path */
	public void printFailure() { }
}

/** Node class to represent cities */
class Node {
	public int id;
	public int pay;
	public boolean visited; // Helps us to keep track of where we've been on the graph
	public String name;
	public Node(int id, int pay){
		this.id = id;
		this.pay = pay;
		this.visited = false;
		this.name = getRandomName();
	}
	public void visit(){
		this.visited = true;
	}
	public void unvisit() {
		this.visited = false;
	}
	public String getRandomName() {
		String sillyName = "";
		try {
			Scanner s = new Scanner(new File("silly-names.dat"));
			Random r = new Random();
			int target = r.nextInt(500);
			for(int i=0;i<target;i++) {
				if(s.hasNextLine()) {
					s.nextLine();
				}
			}
			sillyName = s.nextLine();
		} catch(FileNotFoundException e) {  
			System.err.println("File not found exception: " + e.getMessage());
		}

		return sillyName;
	}	
}

/** Edge class to represent roads connecting cities */
class Edge {
	public int cost;
	public int distance;
	public int MPG;
	public Edge(int cost) {
		this.cost = cost;
		this.MPG = 1;
		this.distance = this.MPG*this.cost;
	}
}
